unit SDCCDLIniFile_U;
// This file is part of Sarah Dean's Code Co-op Diff Launcher.
//
//  "Sarah Dean's Code Co-op Diff Launcher" is free software; you can
// redistribute it and/or modify it under the terms of the GNU General
// Public License as published by the Free Software Foundation; either
// version 2 of the License, or at your option) any later version.
//
//  "Sarah Dean's Code Co-op Diff Launcher" is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY; without even the
// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with "Sarah Dean's Code Co-op Diff Launcher"; if not, write to
// the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
// MA  02111-1307  USA


// Description: SecureTrayUtil INI File
// By Sarah Dean
//
// Defines INI file contents and methods for reading and writing them
//
// -----------------------------------------------------------------------------


interface

uses
     registry,
     dialogs,
     forms,
     classes,
     sysutils,
     Windows, // needed for WritePrivateProfileString
     SDCCDLIniFileConsts_U,
     AppGlobalConsts_U;

type

  TSDCCDLIniFile = class(TRegINIFile)
  private
    function ReadIniFileGeneral() : boolean;

    function WriteIniFileGeneral() : boolean;
  public
    NewVersion: boolean; // Set if this if there is no INI file or if it's for a previous version

    // General options
    OptDiffCommand: string;
    OptDiffParameters: string;
    OptDebug: boolean;

    constructor Create(const FileName : string);
    destructor  Destroy(); override;

    function ReadIniFile() : boolean;
    function WriteIniFile() : boolean;

    procedure CheckVersionID();
  end;

implementation

uses
     SDUGeneral;

constructor TSDCCDLIniFile.Create(const FileName : string);
begin
  inherited;
end;

destructor TSDCCDLIniFile.Destroy();
begin
  inherited;
end;

function TSDCCDLIniFile.ReadIniFile() : boolean;
var
  retVal : boolean;
begin
  retVal := ReadIniFileGeneral();

  Result := retVal;
end;

function TSDCCDLIniFile.ReadIniFileGeneral() : boolean;
begin
  OptDiffCommand := ReadString(SECTION_CODECOOPDIFFLAUNCHER, KEY_DIFFCOMMAND, DFLT_KEY_DIFFCOMMAND);
  OptDiffParameters := ReadString(SECTION_CODECOOPDIFFLAUNCHER, KEY_DIFFPARAMETERS, DFLT_KEY_DIFFPARAMETERS);

  OptDebug := ReadBool(SECTION_CODECOOPDIFFLAUNCHER, KEY_DEBUG, DFLT_KEY_DEBUG);

  Result := TRUE;
end;


function TSDCCDLIniFile.WriteIniFile() : boolean;
var
  retVal : boolean;
begin
  retVal := WriteIniFileGeneral();

  Result := retVal;
end;

function TSDCCDLIniFile.WriteIniFileGeneral() : boolean;
var
  thisMajorVersion   : integer;
  thisMinorVersion   : integer;
  thisRevisionVersion: integer;
  thisBuildVersion   : integer;

  strVersion: string;
begin
  SDUGetVersionInfo('', thisMajorVersion, thisMinorVersion, thisRevisionVersion, thisBuildVersion);
  strVersion := SDUGetVersionInfoString('');
  if (APP_BETA_BUILD>-1) then
    begin
    strVersion := strVersion + ' BETA ' + inttostr(APP_BETA_BUILD);
    end;

  WriteString(SECTION_CODECOOPDIFFLAUNCHER, KEY_VERSION, strVersion);
  WriteInteger(SECTION_CODECOOPDIFFLAUNCHER, KEY_MAJOR_VERSION,   thisMajorVersion);
  WriteInteger(SECTION_CODECOOPDIFFLAUNCHER, KEY_MINOR_VERSION,   thisMinorVersion);
  WriteInteger(SECTION_CODECOOPDIFFLAUNCHER, KEY_RELEASE_VERSION, thisRevisionVersion);
  WriteInteger(SECTION_CODECOOPDIFFLAUNCHER, KEY_BUILD_VERSION,   thisBuildVersion);
  WriteInteger(SECTION_CODECOOPDIFFLAUNCHER, KEY_BETA_VERSION,    APP_BETA_BUILD);


  WriteString(SECTION_CODECOOPDIFFLAUNCHER, KEY_DIFFCOMMAND, OptDiffCommand);
  WriteString(SECTION_CODECOOPDIFFLAUNCHER, KEY_DIFFPARAMETERS, OptDiffParameters);
  WriteBool(SECTION_CODECOOPDIFFLAUNCHER, KEY_DEBUG, OptDebug);


  Result := TRUE;
end;



procedure TSDCCDLIniFile.CheckVersionID();
var
  oldVersionStr: string;
  upgradeMessage: string;

  majorVersion   : integer; // Only used to build up oldVersionStr
  minorVersion   : integer; // Only used to build up oldVersionStr
  revisionVersion: integer; // Only used to build up oldVersionStr
  buildVersion   : integer; // Only used to build up oldVersionStr

  thisMajorVersion   : integer;
  thisMinorVersion   : integer;
  thisRevisionVersion: integer;
  thisBuildVersion   : integer;
  strThisVersion     : string;

  betaVersion: integer;
begin
  oldVersionStr    := ReadString(SECTION_CODECOOPDIFFLAUNCHER, KEY_VERSION, '-1.0');
  majorVersion     := ReadInteger(SECTION_CODECOOPDIFFLAUNCHER, KEY_MAJOR_VERSION, -1);
  minorVersion     := ReadInteger(SECTION_CODECOOPDIFFLAUNCHER, KEY_MINOR_VERSION, -1);
  revisionVersion  := ReadInteger(SECTION_CODECOOPDIFFLAUNCHER, KEY_RELEASE_VERSION, -1);
  buildVersion     := ReadInteger(SECTION_CODECOOPDIFFLAUNCHER, KEY_BUILD_VERSION, -1);
  betaVersion      := ReadInteger(SECTION_CODECOOPDIFFLAUNCHER, KEY_BETA_VERSION, -1);

  SDUGetVersionInfo('', thisMajorVersion, thisMinorVersion, thisRevisionVersion, thisBuildVersion);
  strThisVersion := Format('%d.%.2d.%.2d.%.4d', [thisMajorVersion, thisMinorVersion, thisRevisionVersion, thisBuildVersion]);
  if (APP_BETA_BUILD>-1) then
    begin
    strThisVersion := strThisVersion + ' BETA ' + inttostr(APP_BETA_BUILD);
    end;

  NewVersion := FALSE;

  if oldVersionStr='-1.0' then
    begin
    // Any message to be displayed the 1st time the user runs the application
    // to be placed here
    NewVersion := TRUE;
    end
  else if (oldVersionStr='1.00') OR
          (oldVersionStr='1.10') OR
          (majorVersion<thisMajorVersion) OR
          ((majorVersion=thisMajorVersion) AND (minorVersion<thisMinorVersion)) OR
          ((betaVersion<>-1) AND (not(APP_BETA_BUILD>-1))) OR
          ((betaVersion<>-1) AND (betaVersion<APP_BETA_BUILD)) then
    begin
    // The INI file dates back from an older version, so warn the user
    
    if not((oldVersionStr='1.00') or (oldVersionStr='1.10')) then
      begin
      oldVersionStr := Format('%d.%.2d.%.2d.%.4d', [majorVersion, minorVersion, revisionVersion, buildVersion]);
      end;

    if betaVersion>-1 then
      begin
      oldVersionStr := oldVersionStr + ' BETA ' + inttostr(betaVersion);
      end;

    upgradeMessage := 'It appears as though your configuration dates back to v'+oldVersionStr;
    upgradeMessage := upgradeMessage+CRLF+CRLF+'You may find that some of your options mysteriously revert back to their defaults when running this version (v'+strThisVersion+').';
    upgradeMessage := upgradeMessage+CRLF+CRLF+'This is "normal", and due to two things:';
    upgradeMessage := upgradeMessage+CRLF+'1) the format of the ini file has changed to make it more logical and to take into account improvements in the software';
    upgradeMessage := upgradeMessage+CRLF+'2) lazyness on the part of the author (it wouldn''t be difficult to write a converter, but it''s just too much hassle)';
    upgradeMessage := upgradeMessage+CRLF+CRLF+'To prevent this dialog from reappearing, either delete your existing SecureTrayUtil.ini file (located in your Windows directory) or goto any of the options screens, change an option, change it back, and click "OK" or "Apply"';
    upgradeMessage := upgradeMessage+CRLF+CRLF+'It is recommended that you check your option settings to ensure correct operation.';

    messagedlg(upgradeMessage, mtWarning, [mbOK], 0);

    NewVersion := TRUE;
    end;

end;


END.

